@@ -58,7 +58,6 @@
 #include <QtCore/QSharedData>
 #include <QtCore/QStack>
 #include <QtCore/QMetaEnum> //used for a kDebug() in BacktraceParserGdb::backtraceUsefulness()
-#include <KGlobal>
 
 //BEGIN BacktraceParser
 
@@ -295,6 +294,7 @@ struct BacktraceParserGdb::Private {
     QList<BacktraceLineGdb> m_linesList;
     QList<BacktraceLineGdb> m_linesToRate;
     QStringList m_firstUsefulFunctions;
+    QSet<QString> m_librariesWithMissingDebugSymbols;
     int m_possibleKCrashStart;
     int m_threadsCount;
     bool m_isBelowSignalHandler;
@@ -351,10 +351,16 @@ void BacktraceParserGdb::parseLine(const QString & lineStr)
         d->m_frameZeroAppeared = false; // gdb bug workaround flag, see below
         break;
     case BacktraceLineGdb::SignalHandlerStart:
-        //replace the stack frames of KCrash with a nice message
-        d->m_linesList.erase(d->m_linesList.begin() + d->m_possibleKCrashStart, d->m_linesList.end());
-        d->m_linesList.insert(d->m_possibleKCrashStart, BacktraceLineGdb("[KCrash Handler]\n"));
-        d->m_isBelowSignalHandler = true; //next line is the first below the signal handler
+        if (!d->m_isBelowSignalHandler) {
+            //replace the stack frames of KCrash with a nice message
+            d->m_linesList.erase(d->m_linesList.begin() + d->m_possibleKCrashStart, d->m_linesList.end());
+            d->m_linesList.insert(d->m_possibleKCrashStart, BacktraceLineGdb("[KCrash Handler]\n"));
+            d->m_isBelowSignalHandler = true; //next line is the first below the signal handler
+        } else {
+            //this is not the first time we see a crash handler frame on the same thread,
+            //so we just add it to the list
+            d->m_linesList.append(line);
+        }
         break;
     case BacktraceLineGdb::StackFrame:
         // gdb workaround - (v6.8 at least) - 'thread apply all bt' writes
@@ -463,7 +469,8 @@ static bool lineShouldBeIgnored(const BacktraceLineGdb & line)
     if ( line.libraryName().contains("libc.so")
         || line.libraryName().contains("libstdc++.so")
         || line.functionName().startsWith(QLatin1String("*__GI_")) //glibc2.9 uses *__GI_ as prefix
-        || line.libraryName().contains("libpthread.so") )
+        || line.libraryName().contains("libpthread.so") 
+        || line.libraryName().contains("libglib-2.0.so") )
         return true;
 
     return false;
@@ -511,6 +518,11 @@ BacktraceParser::Usefulness BacktraceParserGdb::backtraceUsefulness() const
             continue;
         }
 
+        if ( line.rating() == BacktraceLineGdb::MissingFunction 
+            || line.rating() == BacktraceLineGdb::MissingSourceFile) {
+            d->m_librariesWithMissingDebugSymbols.insert(line.libraryName().trimmed());
+        }
+        
         if ( line.rating() != BacktraceLineGdb::MissingEverything
             && line.rating() != BacktraceLineGdb::MissingFunction ) {
             usefulFunctionsStack.push(line);
@@ -574,6 +586,11 @@ QStringList BacktraceParserGdb::firstValidFunctions() const
     return d ? d->m_firstUsefulFunctions : QStringList();
 }
 
+QSet<QString> BacktraceParserGdb::librariesWithMissingDebugSymbols() const
+{
+    return d->m_librariesWithMissingDebugSymbols;
+}
+
 //END BacktraceParserGdb
 
 //BEGIN BacktraceParserNull
@@ -606,6 +623,11 @@ void BacktraceParserNull::newLine(const QString & lineStr)
     m_lines.append(lineStr);
 }
 
+QSet<QString> BacktraceParserNull::librariesWithMissingDebugSymbols() const
+{
+    return QSet<QString>();
+}
+
 //END BacktraceParserNull
 
 #include "backtraceparser.moc"
