@@ -380,8 +380,6 @@ bool BugzillaInformationPage::showNextPage()
     checkTexts();
 
     if (m_textsOK) {
-        bool titleShort = ui.m_titleEdit->isVisible() ? (ui.m_titleEdit->text().size() < 30) : false;
-
         //Calculate the minimum number of characters required for a description
         //If creating a new report: minimum 40, maximum 80
         //If attaching to an existant report: minimum 30, maximum 50
@@ -389,20 +387,10 @@ bool BugzillaInformationPage::showNextPage()
         int requiredDescription = 20 + (reportInterface()->selectedOptionsRating() * multiplier);
         bool detailsShort = ui.m_detailsEdit->toPlainText().size() < requiredDescription;
 
-        if (titleShort || detailsShort) {
+        if (detailsShort) {
             //The user input is less than we want.... encourage to write more
-            QString message;
-
-            if (titleShort && !detailsShort) {
-                message = i18nc("@info","The title does not provide enough information.");
-            } else if (detailsShort && !titleShort) {
-                message = i18nc("@info","The description about the crash details does not provide "
+            QString message = i18nc("@info","The description about the crash details does not provide "
                                         "enough information.");
-            } else {
-                message = i18nc("@info","Both the title and the description about the crash "
-                                        "details do not provide enough information.");
-            }
-
             message += ' ' + i18nc("@info","If you cannot provide enough information, your report "
                                     "will probably waste developers' time. Can you tell us more?");
 
