@@ -68,6 +68,7 @@ QScriptValue constructColorClass(QScriptEngine *engine);
 QScriptValue constructEasingCurveClass(QScriptEngine *engine);
 QScriptValue constructFontClass(QScriptEngine *engine);
 QScriptValue constructGraphicsItemClass(QScriptEngine *engine);
+QScriptValue constructIconClass(QScriptEngine *engine);
 QScriptValue constructKUrlClass(QScriptEngine *engine);
 QScriptValue constructLinearLayoutClass(QScriptEngine *engine);
 QScriptValue constructGridLayoutClass(QScriptEngine *engine);
@@ -140,6 +141,7 @@ void SimpleJavaScriptApplet::reportError(QScriptEngine *engine, bool fatal)
         kDebug() << failureMsg;
     }
 
+    kDebug() << "Error in " << engine->uncaughtException().property("fileName").toString() << "on line" << engine->uncaughtExceptionLineNumber();
     kDebug() << engine->uncaughtExceptionBacktrace();
 }
 
@@ -215,7 +217,7 @@ void SimpleJavaScriptApplet::paintInterface(QPainter *p, const QStyleOptionGraph
     QScriptValueList args;
     args << m_engine->toScriptValue(p);
     args << m_engine->toScriptValue(const_cast<QStyleOptionGraphicsItem*>(option));
-    args << m_engine->toScriptValue(contentsRect);
+    args << m_engine->toScriptValue(QRectF(contentsRect));
 
     QScriptContext *ctx = m_engine->pushContext();
     ctx->setActivationObject(m_self);
@@ -428,7 +430,7 @@ QScriptValue SimpleJavaScriptApplet::getUrl(QScriptContext *context, QScriptEngi
         return engine->undefinedValue();
     }
 
-    KIO::Job *job = KIO::get(url);
+    KIO::Job *job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
     return engine->newQObject(job);
 }
 
@@ -534,13 +536,6 @@ void SimpleJavaScriptApplet::setupObjects()
     global.setProperty("AnimationGroup", m_engine->newFunction(SimpleJavaScriptApplet::animationGroup));
     global.setProperty("ParallelAnimationGroup", m_engine->newFunction(SimpleJavaScriptApplet::parallelAnimationGroup));
 
-    // Bindings for data engine
-    m_engine->setDefaultPrototype(qMetaTypeId<DataEngine*>(), m_engine->newQObject(new DataEngine(), QScriptEngine::ScriptOwnership));
-    m_engine->setDefaultPrototype(qMetaTypeId<Service*>(), m_engine->newQObject(new DummyService(), QScriptEngine::ScriptOwnership));
-    m_engine->setDefaultPrototype(qMetaTypeId<ServiceJob*>(),
-                                  m_engine->newQObject(new ServiceJob(QString(), QString(), QMap<QString, QVariant>()),
-                                                       QScriptEngine::ScriptOwnership ));
-
     bindI18N(m_engine);
     global.setProperty("dataEngine", m_engine->newFunction(SimpleJavaScriptApplet::dataEngine));
     global.setProperty("service", m_engine->newFunction(SimpleJavaScriptApplet::service));
@@ -570,7 +565,7 @@ void SimpleJavaScriptApplet::setupObjects()
     fun = m_engine->newFunction(SimpleJavaScriptApplet::print);
     global.setProperty("print", fun);
     fun = m_engine->newFunction(SimpleJavaScriptApplet::debug);
-    global.setProperty("print", debug);
+    global.setProperty("debug", fun);
 
     // Work around bug in 4.3.0
     qMetaTypeId<QVariant>();
@@ -578,6 +573,7 @@ void SimpleJavaScriptApplet::setupObjects()
     // Add stuff from Qt
     global.setProperty("QPainter", constructPainterClass(m_engine));
     global.setProperty("QGraphicsItem", constructGraphicsItemClass(m_engine));
+    global.setProperty("QIcon", constructIconClass(m_engine));
     global.setProperty("QTimer", constructTimerClass(m_engine));
     global.setProperty("QFont", constructFontClass(m_engine));
     global.setProperty("QColor", constructColorClass(m_engine));
@@ -684,7 +680,9 @@ QScriptValue SimpleJavaScriptApplet::animation(QScriptContext *context, QScriptE
                 anim->setTargetWidget(parent);
             }
 
-            return engine->newQObject(anim);
+            QScriptValue value = engine->newQObject(anim);
+            registerEnums(engine, value, *anim->metaObject());
+            return value;
         }
     }
 
